@@ -27,6 +27,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkkeysyms-compat.h>
 
 #include <unistd.h>
 #include <string.h>
@@ -68,6 +69,7 @@ static void show_popup_menu( MainWin* mw, GdkEventButton* evt );
 static gboolean on_delete_event( GtkWidget* widget, GdkEventAny* evt );
 static void on_size_allocate( GtkWidget* widget, GtkAllocation    *allocation );
 static gboolean on_win_state_event( GtkWidget* widget, GdkEventWindowState* state );
+static void on_scroll_size_allocate(GtkWidget* widget, GtkAllocation* allocation, MainWin* mv);
 static void on_zoom_fit( GtkToggleButton* btn, MainWin* mw );
 static void on_zoom_fit_menu( GtkMenuItem* item, MainWin* mw );
 static void on_full_screen( GtkWidget* btn, MainWin* mw );
@@ -88,6 +90,7 @@ static void on_open( GtkWidget* btn, MainWin* mw );
 static void on_zoom_in( GtkWidget* btn, MainWin* mw );
 static void on_zoom_out( GtkWidget* btn, MainWin* mw );
 static void on_preference( GtkWidget* btn, MainWin* mw );
+static void on_toggle_toolbar( GtkMenuItem* item, MainWin* mw );
 static void on_quit( GtkWidget* btn, MainWin* mw );
 static gboolean on_button_press( GtkWidget* widget, GdkEventButton* evt, MainWin* mw );
 static gboolean on_button_release( GtkWidget* widget, GdkEventButton* evt, MainWin* mw );
@@ -108,6 +111,10 @@ void on_flip_horizontal( GtkWidget* btn, MainWin* mw );
 static int trans_angle_to_id(int i);
 static int get_new_angle( int orig_angle, int rotate_angle );
 
+static void main_win_set_zoom_scale(MainWin* mw, double scale);
+static void main_win_set_zoom_mode(MainWin* mw, ZoomMode mode);
+static void main_win_update_zoom_buttons_state(MainWin* mw);
+
 // Begin of GObject-related stuff
 
 G_DEFINE_TYPE( MainWin, main_win, GTK_TYPE_WINDOW )
@@ -137,8 +144,11 @@ void main_win_finalize( GObject* obj )
 
     if( G_LIKELY(mw->img_list) )
         image_list_free( mw->img_list );
+#if GTK_CHECK_VERSION(3, 0, 0)
+    g_object_unref( mw->hand_cursor );
+#else
     gdk_cursor_unref( mw->hand_cursor );
-
+#endif
     // FIXME: Put this here is weird
     gtk_main_quit();
 }
@@ -153,10 +163,21 @@ GtkWidget* main_win_new()
 void main_win_init( MainWin*mw )
 {
     gtk_window_set_title( (GtkWindow*)mw, _("Image Viewer"));
-    gtk_window_set_icon_from_file( (GtkWindow*)mw, PACKAGE_DATA_DIR"/pixmaps/gpicview.png", NULL );
+    if (gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), "gpicview"))
+    {
+        gtk_window_set_icon_name((GtkWindow*)mw, "gpicview");
+    }
+    else
+    {
+        gtk_window_set_icon_from_file((GtkWindow*)mw, PACKAGE_DATA_DIR "/icons/hicolor/48x48/apps/gpicview.png", NULL);
+    }
     gtk_window_set_default_size( (GtkWindow*)mw, 640, 480 );
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    GtkWidget* box = gtk_box_new( GTK_ORIENTATION_VERTICAL, 0 );
+#else
     GtkWidget* box = gtk_vbox_new( FALSE, 0 );
+#endif
     gtk_container_add( (GtkContainer*)mw, box);
 
     // image area
@@ -180,22 +201,34 @@ void main_win_init( MainWin*mw )
     mw->img_view = image_view_new();
     gtk_container_add( (GtkContainer*)mw->evt_box, (GtkWidget*)mw->img_view);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+#else
     const char scroll_style[]=
             "style \"gpicview-scroll\" {"
             "GtkScrolledWindow::scrollbar-spacing=0"
             "}"
             "class \"GtkScrolledWindow\" style \"gpicview-scroll\"";
     gtk_rc_parse_string( scroll_style );
+#endif
     mw->scroll = gtk_scrolled_window_new( NULL, NULL );
+    g_signal_connect(G_OBJECT(mw->scroll), "size-allocate", G_CALLBACK(on_scroll_size_allocate), (gpointer) mw);
     gtk_scrolled_window_set_shadow_type( (GtkScrolledWindow*)mw->scroll, GTK_SHADOW_NONE );
     gtk_scrolled_window_set_policy((GtkScrolledWindow*)mw->scroll,
                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     GtkAdjustment *hadj, *vadj;
     hadj = gtk_scrolled_window_get_hadjustment((GtkScrolledWindow*)mw->scroll);
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gtk_adjustment_set_page_increment(hadj, 10);
+#else
     hadj->page_increment = 10;
+#endif
     gtk_adjustment_changed(hadj);
     vadj = gtk_scrolled_window_get_vadjustment((GtkScrolledWindow*)mw->scroll);
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gtk_adjustment_set_page_increment(vadj, 10);
+#else
     vadj->page_increment = 10;
+#endif
     gtk_adjustment_changed(vadj);
 
     image_view_set_adjustments( IMAGE_VIEW(mw->img_view), hadj, vadj );    // dirty hack :-(
@@ -210,6 +243,12 @@ void main_win_init( MainWin*mw )
     create_nav_bar( mw, box );
     gtk_widget_show_all( box );
 
+    if (pref.show_toolbar)
+        gtk_widget_show(gtk_widget_get_parent(mw->nav_bar));
+    else
+        gtk_widget_hide(gtk_widget_get_parent(mw->nav_bar));
+
+
     mw->hand_cursor = gdk_cursor_new_for_display( gtk_widget_get_display((GtkWidget*)mw), GDK_FLEUR );
 
 //    zoom_mode = ZOOM_NONE;
@@ -333,6 +372,16 @@ static void update_btns(MainWin* mw)
 
 gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
 {
+    if (g_file_test(file_path, G_FILE_TEST_IS_DIR))
+    {
+        image_list_open_dir( mw->img_list, file_path, NULL );
+        image_list_sort_by_name( mw->img_list, GTK_SORT_DESCENDING );
+        if (image_list_get_first(mw->img_list))
+            main_win_open(mw, image_list_get_current_file_path(mw->img_list), zoom);
+        return;
+    }
+
+
     GError* err = NULL;
     GdkPixbufFormat* info;
     info = gdk_pixbuf_get_file_info( file_path, NULL, NULL );
@@ -410,18 +459,14 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
 
     if( mw->zoom_mode == ZOOM_FIT )
     {
-        gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, TRUE );
         main_win_fit_window_size( mw, FALSE, GDK_INTERP_BILINEAR );
     }
     else  if( mw->zoom_mode == ZOOM_SCALE )  // scale
     {
-        gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, FALSE );
-        gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, FALSE );
         main_win_scale_image( mw, mw->scale, GDK_INTERP_BILINEAR );
     }
     else  if( mw->zoom_mode == ZOOM_ORIG )  // original size
     {
-        gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, TRUE );
         image_view_set_scale( (ImageView*)mw->img_view, mw->scale, GDK_INTERP_BILINEAR );
         main_win_center_image( mw );
     }
@@ -446,6 +491,8 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
     update_title( disp_name, mw );
     g_free( disp_name );
 
+    main_win_update_zoom_buttons_state(mw);
+
     return TRUE;
 }
 
@@ -508,7 +555,7 @@ void on_size_allocate( GtkWidget* widget, GtkAllocation    *allocation )
 gboolean on_win_state_event( GtkWidget* widget, GdkEventWindowState* state )
 {
     MainWin* mw = (MainWin*)widget;
-    if( state->new_window_state == GDK_WINDOW_STATE_FULLSCREEN )
+    if( (state->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0 )
     {
         gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &pref.bg_full );
         gtk_widget_hide( gtk_widget_get_parent(mw->nav_bar) );
@@ -517,7 +564,8 @@ gboolean on_win_state_event( GtkWidget* widget, GdkEventWindowState* state )
     else
     {
         gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &pref.bg );
-        gtk_widget_show( gtk_widget_get_parent(mw->nav_bar) );
+        if (pref.show_toolbar)
+            gtk_widget_show( gtk_widget_get_parent(mw->nav_bar) );
         mw->full_screen = FALSE;
     }
 
@@ -559,8 +607,7 @@ void main_win_fit_window_size(  MainWin* mw, gboolean can_strech, GdkInterpType
 
     if( mw->pix == NULL )
         return;
-
-    main_win_fit_size( mw, mw->scroll->allocation.width, mw->scroll->allocation.height, can_strech, type );
+    main_win_fit_size( mw, mw->scroll_allocation.width, mw->scroll_allocation.height, can_strech, type );
 }
 
 GtkWidget* add_nav_btn( MainWin* mw, const char* icon, const char* tip, GCallback cb, gboolean toggle)
@@ -596,6 +643,11 @@ GtkWidget* add_nav_btn_img( MainWin* mw, const char* icon, const char* tip, GCal
     return btn;
 }
 
+void on_scroll_size_allocate(GtkWidget* widget, GtkAllocation* allocation, MainWin* mv)
+{
+    mv->scroll_allocation = *allocation;
+}
+
 void on_zoom_fit_menu( GtkMenuItem* item, MainWin* mw )
 {
     gtk_button_clicked( (GtkButton*)mw->btn_fit );
@@ -603,15 +655,8 @@ void on_zoom_fit_menu( GtkMenuItem* item, MainWin* mw )
 
 void on_zoom_fit( GtkToggleButton* btn, MainWin* mw )
 {
-    if( ! btn->active )
-    {
-        if( mw->zoom_mode == ZOOM_FIT )
-            gtk_toggle_button_set_active( btn, TRUE );
-        return;
-    }
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, FALSE );
-
-    main_win_fit_window_size( mw, FALSE, GDK_INTERP_BILINEAR );
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)))
+        main_win_set_zoom_mode(mw, ZOOM_FIT);
 }
 
 void on_full_screen( GtkWidget* btn, MainWin* mw )
@@ -636,31 +681,8 @@ void on_orig_size( GtkToggleButton* btn, MainWin* mw )
         return;
     }
 
-    if( ! btn->active )
-    {
-        if( mw->zoom_mode == ZOOM_ORIG )
-            gtk_toggle_button_set_active( btn, TRUE );
-        return;
-    }
-    mw->zoom_mode = ZOOM_ORIG;
-    mw->scale = 1.0;
-//    gtk_scrolled_window_set_policy( (GtkScrolledWindow*)mw->scroll,
-//                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, FALSE );
-
-    if( ! mw->pix )
-        return;
-
-    // update scale
-    update_title(NULL, mw);
-
-    image_view_set_scale( (ImageView*)mw->img_view, 1.0, GDK_INTERP_BILINEAR );
-
-    while (gtk_events_pending ())
-        gtk_main_iteration ();
-
-    main_win_center_image( mw ); // FIXME:  mw doesn't work well. Why?
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)))
+        main_win_set_zoom_mode(mw, ZOOM_ORIG);
 }
 
 void on_prev( GtkWidget* btn, MainWin* mw )
@@ -680,7 +702,7 @@ void on_prev( GtkWidget* btn, MainWin* mw )
     if( name )
     {
         char* file_path = image_list_get_current_file_path( mw->img_list );
-        main_win_open( mw, file_path, ZOOM_FIT );
+        main_win_open( mw, file_path, mw->zoom_mode );
         g_free( file_path );
     }
 }
@@ -701,7 +723,7 @@ void on_next( GtkWidget* btn, MainWin* mw )
     if( name )
     {
         char* file_path = image_list_get_current_file_path( mw->img_list );
-        main_win_open( mw, file_path, ZOOM_FIT );
+        main_win_open( mw, file_path, mw->zoom_mode );
         g_free( file_path );
     }
 }
@@ -909,47 +931,16 @@ void on_open( GtkWidget* btn, MainWin* mw )
 
 void on_zoom_in( GtkWidget* btn, MainWin* mw )
 {
-    mw->zoom_mode = ZOOM_SCALE;
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, FALSE );
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, FALSE );
-//    gtk_scrolled_window_set_policy( (GtkScrolledWindow*)mw->scroll,
-//                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-
     double scale = mw->scale;
-    if( mw->pix && scale < 20.0 )
-    {
-//        busy(TRUE);
-            scale *= 1.05;
-            if( scale > 20.0 )
-                scale = 20.0;
-            if( mw->scale != scale )
-                main_win_scale_image( mw, scale, GDK_INTERP_BILINEAR );
-//        adjust_adjustment_on_zoom(oldscale);
-//        busy(FALSE);
-    }
+    scale *= 1.05;
+    main_win_set_zoom_scale(mw, scale);
 }
 
 void on_zoom_out( GtkWidget* btn, MainWin* mw )
 {
-    mw->zoom_mode = ZOOM_SCALE;
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, FALSE );
-    gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, FALSE );
-//    gtk_scrolled_window_set_policy( (GtkScrolledWindow*)mw->scroll,
-//                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-
     double scale = mw->scale;
-    if( mw->pix && scale > 0.02 )
-    {
-//        busy(TRUE);
-
-        scale /= 1.05;
-        if( scale < 0.02 )
-            scale = 0.02;
-        if( mw->scale != scale )
-            main_win_scale_image( mw, scale, GDK_INTERP_BILINEAR );
-//        adjust_adjustment_on_zoom(oldscale);
-//        busy(FALSE);
-    }
+    scale /= 1.05;
+    main_win_set_zoom_scale(mw, scale);
 }
 
 void on_preference( GtkWidget* btn, MainWin* mw )
@@ -980,7 +971,7 @@ gboolean on_button_press( GtkWidget* widget, GdkEventButton* evt, MainWin* mw )
                 return FALSE;
             mw->dragging = TRUE;
             gtk_widget_get_pointer( (GtkWidget*)mw, &mw->drag_old_x ,&mw->drag_old_y );
-            gdk_window_set_cursor( widget->window, mw->hand_cursor );
+            gdk_window_set_cursor( gtk_widget_get_window(widget), mw->hand_cursor );
         }
         else if( evt->button == 3 )   // right button
         {
@@ -1012,35 +1003,57 @@ gboolean on_mouse_move( GtkWidget* widget, GdkEventMotion* evt, MainWin* mw )
     GtkRequisition req;
     gtk_widget_size_request( (GtkWidget*)mw->img_view, &req );
 
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gdouble hadj_page_size = gtk_adjustment_get_page_size(hadj);
+    gdouble hadj_lower = gtk_adjustment_get_lower(hadj);
+    gdouble hadj_upper = gtk_adjustment_get_upper(hadj);
+#else
+    gdouble hadj_page_size = hadj->page_size;
+    gdouble hadj_lower = hadj->lower;
+    gdouble hadj_upper = hadj->upper;
+#endif
+
     if( ABS(dx) > 4 )
     {
         mw->drag_old_x = cur_x;
-        if( req.width > hadj->page_size )
+        if( req.width > hadj_page_size )
         {
-            gdouble x = gtk_adjustment_get_value (hadj) + dx;
-            if( x < hadj->lower )
-                x = hadj->lower;
-            else if( (x + hadj->page_size) > hadj->upper )
-                x = hadj->upper - hadj->page_size;
+            gdouble value = gtk_adjustment_get_value (hadj);
+            gdouble x = value + dx;
+            if( x < hadj_lower )
+                x = hadj_lower;
+            else if( (x + hadj_page_size) > hadj_upper )
+                x = hadj_upper - hadj_page_size;
 
-            if( x != hadj->value )
+            if( x != value )
                 gtk_adjustment_set_value (hadj, x );
         }
     }
 
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gdouble vadj_page_size = gtk_adjustment_get_page_size(vadj);
+    gdouble vadj_lower = gtk_adjustment_get_lower(vadj);
+    gdouble vadj_upper = gtk_adjustment_get_upper(vadj);
+#else
+    gdouble vadj_page_size = vadj->page_size;
+    gdouble vadj_lower = vadj->lower;
+    gdouble vadj_upper = vadj->upper;
+#endif
+
     if( ABS(dy) > 4 )
     {
-        if( req.height > vadj->page_size )
+        if( req.height > vadj_page_size )
         {
             mw->drag_old_y = cur_y;
-            gdouble y = gtk_adjustment_get_value (vadj) + dy;
-            if( y < vadj->lower )
-                y = vadj->lower;
-            else if( (y + vadj->page_size) > vadj->upper )
-                y = vadj->upper - vadj->page_size;
+            gdouble value = gtk_adjustment_get_value (vadj);
+            gdouble y = value + dy;
+            if( y < vadj_lower )
+                y = vadj_lower;
+            else if( (y + vadj_page_size) > vadj_upper )
+                y = vadj_upper - vadj_page_size;
 
-            if( y != vadj->value )
-                gtk_adjustment_set_value (vadj, y  );
+            if( y != value )
+                gtk_adjustment_set_value (vadj, y );
         }
     }
     return FALSE;
@@ -1049,7 +1062,7 @@ gboolean on_mouse_move( GtkWidget* widget, GdkEventMotion* evt, MainWin* mw )
 gboolean on_button_release( GtkWidget* widget, GdkEventButton* evt, MainWin* mw )
 {
     mw->dragging = FALSE;
-    gdk_window_set_cursor( widget->window, NULL );
+    gdk_window_set_cursor( gtk_widget_get_window(widget), NULL );
     return FALSE;
 }
 
@@ -1182,6 +1195,10 @@ gboolean on_key_press_event(GtkWidget* widget, GdkEventKey * key)
         case GDK_P:
             on_preference( NULL, mw );
 	    break;
+        case GDK_t:
+        case GDK_T:
+            on_toggle_toolbar( NULL, mw );
+	    break;
         case GDK_Escape:
             if( mw->full_screen )
                 on_full_screen( NULL, mw );
@@ -1211,11 +1228,27 @@ void main_win_center_image( MainWin* mw )
     GtkRequisition req;
     gtk_widget_size_request( (GtkWidget*)mw->img_view, &req );
 
-    if( req.width > hadj->page_size )
-        gtk_adjustment_set_value(hadj, ( hadj->upper - hadj->page_size ) / 2 );
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gdouble hadj_page_size = gtk_adjustment_get_page_size(hadj);
+    gdouble hadj_upper = gtk_adjustment_get_upper(hadj);
+#else
+    gdouble hadj_page_size = hadj->page_size;
+    gdouble hadj_upper = hadj->upper;
+#endif
 
-    if( req.height > vadj->page_size )
-        gtk_adjustment_set_value(vadj, ( vadj->upper - vadj->page_size ) / 2 );
+    if( req.width > hadj_page_size )
+        gtk_adjustment_set_value(hadj, ( hadj_upper - hadj_page_size ) / 2 );
+
+#if GTK_CHECK_VERSION(2, 14, 0)
+    gdouble vadj_page_size = gtk_adjustment_get_page_size(vadj);
+    gdouble vadj_upper = gtk_adjustment_get_upper(vadj);
+#else
+    gdouble vadj_page_size = vadj->page_size;
+    gdouble vadj_upper = vadj->upper;
+#endif
+
+    if( req.height > vadj_page_size )
+        gtk_adjustment_set_value(vadj, ( vadj_upper - vadj_page_size ) / 2 );
 }
 
 void rotate_image( MainWin* mw, int angle )
@@ -1392,6 +1425,19 @@ void on_delete( GtkWidget* btn, MainWin* mw )
     }
 }
 
+void on_toggle_toolbar( GtkMenuItem* item, MainWin* mw )
+{
+    pref.show_toolbar = !pref.show_toolbar;
+
+    if (pref.show_toolbar)
+        gtk_widget_show(gtk_widget_get_parent(mw->nav_bar));
+    else
+        gtk_widget_hide(gtk_widget_get_parent(mw->nav_bar));
+
+    save_preferences();
+}
+
+
 void show_popup_menu( MainWin* mw, GdkEventButton* evt )
 {
     static PtkMenuItemEntry menu_def[] =
@@ -1419,6 +1465,7 @@ void show_popup_menu( MainWin* mw, GdkEventButton* evt )
         PTK_IMG_MENU_ITEM( N_("Delete File"), GTK_STOCK_DELETE, G_CALLBACK(on_delete), GDK_Delete, 0 ),
         PTK_SEPARATOR_MENU_ITEM,
         PTK_IMG_MENU_ITEM( N_("Preferences"), GTK_STOCK_PREFERENCES, G_CALLBACK(on_preference), GDK_P, 0 ),
+        PTK_IMG_MENU_ITEM( N_("Show/Hide toolbar"), NULL, G_CALLBACK(on_toggle_toolbar), GDK_T, 0 ),
         PTK_STOCK_MENU_ITEM( GTK_STOCK_ABOUT, on_about ),
         PTK_SEPARATOR_MENU_ITEM,
         PTK_IMG_MENU_ITEM( N_("Quit"), GTK_STOCK_QUIT, G_CALLBACK(on_quit), GDK_Q, 0 ),
@@ -1451,28 +1498,6 @@ void show_popup_menu( MainWin* mw, GdkEventButton* evt )
     gtk_menu_popup( (GtkMenu*)popup, NULL, NULL, NULL, NULL, evt->button, evt->time );
 }
 
-/* callback used to open default browser when URLs got clicked */
-static void open_url( GtkAboutDialog *dlg, const gchar *url, gpointer data)
-{
-    /* FIXME: is there any better way to do this? */
-    char* programs[] = { "xdg-open", "gnome-open", "exo-open" };
-    int i;
-    for(  i = 0; i < G_N_ELEMENTS(programs); ++i)
-    {
-        gchar* open_cmd = NULL;
-        if( (open_cmd = g_find_program_in_path( programs[i] )) )
-        {
-             char* argv [3];
-             argv [0] = programs[i];
-             argv [1] = (gchar *) url;
-             argv [2] = NULL;
-             g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
-             g_free( open_cmd );
-             break;
-        }
-    }
-}
-
 void on_about( GtkWidget* menu, MainWin* mw )
 {
     GtkWidget * about_dlg;
@@ -1495,7 +1520,7 @@ void on_about( GtkWidget* menu, MainWin* mw )
     gtk_container_set_border_width ( ( GtkContainer*)about_dlg , 2 );
     gtk_about_dialog_set_version ( (GtkAboutDialog*)about_dlg, VERSION );
     gtk_about_dialog_set_program_name ( (GtkAboutDialog*)about_dlg, _( "GPicView" ) );
-    gtk_about_dialog_set_logo( (GtkAboutDialog*)about_dlg, gdk_pixbuf_new_from_file(  PACKAGE_DATA_DIR"/pixmaps/gpicview.png", NULL ) );
+    gtk_about_dialog_set_logo_icon_name ( (GtkAboutDialog*)about_dlg, "gpicview" );
     gtk_about_dialog_set_copyright ( (GtkAboutDialog*)about_dlg, _( "Copyright (C) 2007 - 2011" ) );
     gtk_about_dialog_set_comments ( (GtkAboutDialog*)about_dlg, _( "Lightweight image viewer from LXDE project" ) );
     gtk_about_dialog_set_license ( (GtkAboutDialog*)about_dlg, "GPicView\n\nCopyright (C) 2007 Hong Jen Yee (PCMan)\n\nThis program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with this program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA." );
@@ -1511,7 +1536,16 @@ void on_about( GtkWidget* menu, MainWin* mw )
 void on_drag_data_received( GtkWidget* widget, GdkDragContext *drag_context,
                 int x, int y, GtkSelectionData* data, guint info, guint time, MainWin* mw )
 {
-    if( ! data || data->length <= 0)
+    if( !data)
+        return;
+
+#if GTK_CHECK_VERSION(2, 14, 0)
+    int data_length = gtk_selection_data_get_length(data);
+#else
+    int data_length = data->length;
+#endif
+
+    if( data_length <= 0)
         return;
 
     char* file = NULL;
@@ -1534,3 +1568,60 @@ void on_drag_data_received( GtkWidget* widget, GdkDragContext *drag_context,
         g_free( file );
     }
 }
+
+static void main_win_set_zoom_scale(MainWin* mw, double scale)
+{
+    main_win_set_zoom_mode(mw, ZOOM_SCALE);
+
+    if (scale > 20.0)
+        scale = 20.0;
+    if (scale < 0.02)
+        scale = 0.02;
+
+    if (mw->scale != scale)
+        main_win_scale_image(mw, scale, GDK_INTERP_BILINEAR);
+}
+
+static void main_win_set_zoom_mode(MainWin* mw, ZoomMode mode)
+{
+    if (mw->zoom_mode == mode)
+       return;
+
+    mw->zoom_mode = mode;
+
+    main_win_update_zoom_buttons_state(mw);
+
+    if (mode == ZOOM_ORIG)
+    {
+        mw->scale = 1.0;
+        if (!mw->pix)
+           return;
+
+        update_title(NULL, mw);
+
+        image_view_set_scale( (ImageView*)mw->img_view, 1.0, GDK_INTERP_BILINEAR );
+
+        while (gtk_events_pending ())
+            gtk_main_iteration ();
+
+        main_win_center_image( mw ); // FIXME:  mw doesn't work well. Why?
+    }
+    else if (mode == ZOOM_FIT)
+    {
+        main_win_fit_window_size( mw, FALSE, GDK_INTERP_BILINEAR );
+    }
+}
+
+static void main_win_update_zoom_buttons_state(MainWin* mw)
+{
+    gboolean button_fit_active = mw->zoom_mode == ZOOM_FIT;
+    gboolean button_orig_active = mw->zoom_mode == ZOOM_ORIG;
+
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->btn_fit)) != button_fit_active)
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->btn_fit), button_fit_active);
+
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->btn_orig)) != button_orig_active)
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->btn_orig), button_orig_active);
+}
+
+
